import unittest

import tensorflow as tf

from transformers_keras.tokenizers import TransformerDefaultTokenizer

from .transformer_tfrecord_generator import TransformerTFRecordGenerator


class TransformerTFRecordGeneratorTest(unittest.TestCase):

    def testGenerateTransformerTFRecordFile(self):
        src_vocab_file = 'testdata/vocab_src.txt'
        tgt_vocab_file = 'testdata/vocab_tgt.txt'
        src_tokenizer = TransformerDefaultTokenizer(
            vocab_file=src_vocab_file,
            do_basic_tokenization=True,
            do_lower_case=True,
            tokenize_chinese_chars=True,
            nerver_split=None,
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>',
        )
        tgt_tokenizer = TransformerDefaultTokenizer(
            vocab_file=tgt_vocab_file,
            do_basic_tokenization=True,
            do_lower_case=True,
            tokenize_chinese_chars=True,
            nerver_split=None,
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>',
        )
        g = TransformerTFRecordGenerator(
            src_tokenizer,
            tgt_tokenizer,
            src_max_len=16,
            tgt_max_len=16,
            record_option='GZIP',
        )
        input_files = [
            'testdata/train.txt'
        ]
        output_files = [
            'testdata/transformer_train.tfrecord'
        ]
        g.generate(input_files, output_files)

    def testReadTransformerTFRecordFile(self):
        f = 'testdata/transformer_train.tfrecord'
        dataset = tf.data.TFRecordDataset(f, compression_type='GZIP')
        dataset = dataset.repeat(10)
        for d in dataset.take(1):
            print(d)

        name_to_features = {
            'src_ids': tf.io.FixedLenFeature([16], tf.int64),
            'tgt_ids': tf.io.FixedLenFeature([16], tf.int64)
        }

        def _parse_fn(record):
            example = tf.io.parse_single_example(record, name_to_features)
            return example

        dataset = dataset.map(lambda x: _parse_fn(x))
        dataset = dataset.batch(2)
        for d in dataset.take(1):
            print(d)


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np
import tensorflow as tf

from transformers_keras.datasets import TransformerTextFileDatasetBuilder
from transformers_keras.tokenizers import TransformerDefaultTokenizer

from .modeling_transformer import (PositionalEncoding, Transformer,
                                   TransformerDecoder, TransformerEmbedding,
                                   TransformerEncoder)


class ModelingTransformerTest(tf.test.TestCase):

    def testPositionalEncoding(self):
        pe = PositionalEncoding(512, 512)
        x = np.random.randint(0, 512, size=(2, 16))
        x = tf.constant(x, shape=(2, 16), dtype=tf.int64)

        encoding = pe(x)
        self.assertEqual([1, 16, 512], encoding.shape)  # shape[0] will be broadcasted when add token embedding

        y = np.random.randint(0, 512, size=(2, 16))
        y = tf.constant(y, shape=(2, 16), dtype=tf.int64)
        encoding2 = pe(y)
        self.assertEqual([1, 16, 512], encoding2.shape)
        self.assertAllEqual(encoding.numpy(), encoding2.numpy())

    def testEmbedding(self):
        te = TransformerEmbedding(100, 128, 128)
        x = np.random.randint(0, 100, size=(2, 16))
        x = tf.constant(x, shape=(2, 16), dtype=tf.int64)
        embedding = te(x)
        self.assertEqual([2, 16, 128], embedding.shape)

        y = x[:, :]
        embedding2 = te(y)
        self.assertEqual([2, 16, 128], embedding2.shape)
        self.assertAllEqual(embedding, embedding2)

    def testEncoder(self):
        encoder = TransformerEncoder(vocab_size=100, num_layers=2)
        inputs = tf.constant([[i for i in range(10)] * 16], shape=(16, 10), dtype=tf.int64)
        output, attn_weights = encoder(inputs=(inputs, None))

        self.assertEqual(output.shape, [16, 10, encoder.hidden_size])  # (batch_size, input_seq_len, d_model)

        # each layers attention weights
        for v in attn_weights:
            self.assertEqual(v.shape, [16, encoder.num_attention_heads, 10, 10])

    def testDecoder(self):
        decoder = TransformerDecoder(vocab_size=100, num_layers=2)
        y = tf.constant(np.random.randint(0, 10, size=(2, 43)), shape=(2, 43), dtype=tf.int64)
        enc_outputs = tf.random.uniform((2, 50, decoder.hidden_size))
        inputs = (y, enc_outputs, None, None)
        output, self_attn_weights, context_attn_weights = decoder(inputs=inputs)

        self.assertEqual(output.shape, [2, 43, decoder.hidden_size])
        for attn in self_attn_weights:
            self.assertEqual(attn.shape, [2, decoder.num_attention_heads, 43, 43])
        for attn in context_attn_weights:
            self.assertEqual(attn.shape, [2, decoder.num_attention_heads, 43, 50])

    def buildTransformerModel(self, config):
        x = tf.keras.layers.Input(shape=(32,), dtype=tf.int32, name='x')
        y = tf.keras.layers.Input(shape=(32,), dtype=tf.int32, name='y')
        model = Transformer(
            src_vocab_size=config.get('src_vocab_size', 100),
            tgt_vocab_size=config.get('tgt_vocab_size', 100),
            max_positions=config.get('max_positions', 512),
            hidden_size=config.get('hidden_size', 512),
            num_encoder_layers=config.get('num_encoder_layers', 2),
            num_decoder_layers=config.get('num_decoder_layers', 2),
            num_attention_heads=config.get('num_attention_heads', 8),
            ffn_size=config.get('ffn_size', 2048),
            dropout_rate=config.get('dropout_rate', 0.2),
            epsilon=config.get('epsilon', 1e-6)
        )
        logits, _, _, _ = model(inputs=(x, y))
        probs = tf.keras.layers.Lambda(lambda x: tf.nn.softmax(x), name='probs')(logits)

        model = tf.keras.Model(inputs=[x, y], outputs=[probs])

        model.compile(
            optimizer='adam',  # TODO(zhouyang.luo) schedule learning rate
            loss={
                'probs': tf.keras.losses.SparseCategoricalCrossentropy(name='loss', from_logits=False),
            },
            metrics={
                'probs': [
                    tf.keras.metrics.SparseCategoricalAccuracy(name='acc'),
                ]
            }
        )

        model.summary()
        return model

    def buildTokenizers(self):
        src_tokenizer = TransformerDefaultTokenizer(
            vocab_file='testdata/vocab_src.txt',
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>'
        )
        tgt_tokenizer = TransformerDefaultTokenizer(
            vocab_file='testdata/vocab_tgt.txt',
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>'
        )
        return src_tokenizer, tgt_tokenizer

    def testTransformerModel(self):
        src_tokenizer, tgt_tokenizer = self.buildTokenizers()
        data_config = {
            'train_shuffle_buffer_size': 100,
            'valid_shuffle_buffer_size': 100,
            'train_repeat_count': 100,
            'train_batch_size': 2,
            'valid_batch_size': 2,
        }
        dataset_builder = TransformerTextFileDatasetBuilder(src_tokenizer, tgt_tokenizer, **data_config)

        model = Transformer(
            src_vocab_size=src_tokenizer.vocab_size,
            tgt_vocab_size=tgt_tokenizer.vocab_size,
            num_encoder_layers=2,
            num_decoder_layers=2,
        )
        train_dataset = dataset_builder.build_train_dataset(
            train_files=[('testdata/train.src.txt', 'testdata/train.tgt.txt')])
        train_dataset = train_dataset.map(lambda x, y: (x[0], x[1]))
        print(next(iter(train_dataset)))

        for i, v in enumerate(train_dataset):
            outputs, _, _, _ = model(v)

    def testTraining(self):
        src_tokenizer = TransformerDefaultTokenizer(
            vocab_file='testdata/vocab_src.txt',
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>'
        )
        tgt_tokenizer = TransformerDefaultTokenizer(
            vocab_file='testdata/vocab_tgt.txt',
            pad_token='<pad>', unk_token='<unk>', sos_token='<s>', eos_token='</s>'
        )
        data_config = {
            'train_shuffle_buffer_size': 100,
            'valid_shuffle_buffer_size': 100,
            'train_repeat_count': 100,
            'train_batch_size': 2,
            'valid_batch_size': 2,
        }
        dataset_builder = TransformerTextFileDatasetBuilder(src_tokenizer, tgt_tokenizer, **data_config)
        model_config = {
            'src_vocab_size': src_tokenizer.vocab_size,
            'tgt_vocab_size': tgt_tokenizer.vocab_size,
        }
        model = self.buildTransformerModel(model_config)

        train_dataset = dataset_builder.build_train_dataset(
            train_files=[('testdata/train.src.txt', 'testdata/train.tgt.txt')])
        print(next(iter(train_dataset)))
        valid_dataset = dataset_builder.build_valid_dataset(
            valid_files=[('testdata/train.src.txt', 'testdata/train.tgt.txt')])
        model.fit(
            train_dataset,
            validation_data=valid_dataset
        )


if __name__ == "__main__":
    unittest.main()



import logging

from transformers_keras.datasets.abstract_dataset_builder import AbstractDatasetBuilder
from transformers_keras.datasets.bert_dataset_builder import BertTFRecordDatasetBuilder
from transformers_keras.datasets.transformer_dataset_builder import TransformerDatasetBuilder
from transformers_keras.datasets.transformer_dataset_builder import TransformerTFRecordDatasetBuilder
from transformers_keras.datasets.transformer_dataset_builder import TransformerTextFileDatasetBuilder
from transformers_keras.tokenizers.bert_tokenizer import BertAbstractTokenizer
from transformers_keras.tokenizers.bert_tokenizer import BertDefaultTokenizer
from transformers_keras.tokenizers.bert_tokenizer import BertVocabBasedTokenizer
from transformers_keras.tokenizers.tokenizer import WordpieceTokenizer, BasicTokenizer
from transformers_keras.tokenizers.transformer_tokenizer import TransformerAbstractTokenizer
from transformers_keras.tokenizers.transformer_tokenizer import TransformerDefaultTokenizer
from transformers_keras.tokenizers.transformer_tokenizer import TransformerJiebaTokenizer
from transformers_keras.tokenizers.transformer_tokenizer import TransformerVocabBasedTokenizer
from .callbacks import SavedModelExporter, TransformerLearningRate
from .layers import DecoderLayer
from .layers import EncoderLayer
from .layers import MultiHeadAttention
from .layers import PointWiseFeedForwardNetwork
from .layers import ScaledDotProductAttention
from .losses import MaskedSparseCategoricalCrossentropy
from .metrics import MaskedSparseCategoricalAccuracy
from .modeling_albert import AlbertEmbedding, AlbertEncoder, AlbertEncoderGroup, AlbertMLMHead
from .modeling_albert import AlbertEncoderLayer, Albert4PreTraining, AlbertModel, AlbertSOPHead
from .modeling_bert import BertEmbedding, BertMLMHead, BertNSPHead, Bert4PreTraining, BertModel
from .modeling_transformer import PositionalEncoding, TransformerEmbedding
from .modeling_transformer import TransformerEncoder, TransformerDecoder, Transformer
from .runners import AlbertRunner, BertRunner, TransformerRunner

__name__ = 'transformers_keras'
r_version__ = '0.1.2'

logging.basicConfig(format="%(asctime)s %(levelname)s %(filename)15s %(lineno)4d] %(message)s", level=logging.INFO)



import abc
import logging
import os

import jieba

from .tokenizer import BasicTokenizer, WordpieceTokenizer


class TransformerAbstractTokenizer(abc.ABC):

    def __init__(self,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 sos_token='[SOS]',
                 eos_token='[EOS]',
                 **kwargs):
        super().__init__()
        self.pad_token = pad_token
        self.unk_token = unk_token
        self.sos_token = sos_token
        self.eos_token = eos_token
        self.pad_id = 0
        self.unk_id = 1
        self.sos_id = 2
        self.eos_id = 3

    def tokenize(self, sequence):
        raise NotImplementedError()

    def encode(self, sequence):
        raise NotImplementedError()

    def decode(self, sequence):
        raise NotImplementedError()

    @property
    def vocab_size(self):
        raise NotImplementedError()

    @property
    def special_tokens(self):
        return [self.pad_token, self.unk_token, self.sos_token, self.eos_token]

    @property
    def special_ids(self):
        return [self.pad_id, self.unk_id, self.sos_id, self.eos_id]

    @property
    def special_token_and_ids(self):
        return [(self.pad_token, self.pad_id),
                (self.unk_token, self.unk_id),
                (self.sos_token, self.sos_id),
                (self.eos_token, self.eos_id)]


class TransformerVocabBasedTokenizer(TransformerAbstractTokenizer):

    def __init__(self, vocab_file, **kwargs):
        super().__init__(**kwargs)
        _sorted_special_tokens = [x[0] for x in sorted(self.special_token_and_ids, key=lambda x:x[1])]
        self.vocab = _sorted_special_tokens + self._read_vocabs(vocab_file)
        self.token2id = self._build_index(self.vocab)
        self.id2token = self._build_reverse_index(self.token2id)

    def _read_vocabs(self, file):
        vocabs = []
        with open(file, mode='rt', encoding='utf8') as fin:
            for line in fin:
                line = line.strip('\n').strip()
                if not line:
                    continue
                if line in self.special_tokens:
                    continue
                vocabs.append(line)
        return vocabs

    def _build_index(self, vocab):
        m = {}
        for i, v in enumerate(vocab):
            m[v] = i
        return m

    def _build_reverse_index(self, token2id):
        m = {}
        for k, v in token2id.items():
            m[v] = k
        return m

    def tokenize(self, sequence):
        raise NotImplementedError()

    def encode(self, sequence):
        tokens = self.tokenize(sequence)
        return [self.token2id.get(t, self.unk_id) for t in tokens]

    def decode(self, sequence):
        return [self.id2token.get(_id, self.unk_token) for _id in sequence]

    @property
    def vocab_size(self):
        return len(self.token2id)


class TransformerDefaultTokenizer(TransformerVocabBasedTokenizer):

    def __init__(self,
                 vocab_file,
                 do_basic_tokenization=True,
                 do_lower_case=True,
                 nerver_split=None,
                 tokenize_chinese_chars=True,
                 max_input_chars_per_word=100,
                 **kwargs):
        super(TransformerDefaultTokenizer, self).__init__(vocab_file, **kwargs)
        self.do_basic_tokenization = do_basic_tokenization
        self.do_lower_case = do_lower_case
        self.never_split = nerver_split
        self.tokenize_chinese_chars = tokenize_chinese_chars
        if self.do_basic_tokenization:
            self.basic_tokenizer = BasicTokenizer(
                do_lower_case=self.do_lower_case,
                never_split=self.never_split,
                tokenize_chinese_chars=self.tokenize_chinese_chars
            )
        else:
            self.basic_tokenizer = None

        self.max_input_chars_per_word = kwargs.get('max_input_chars_per_word', 100)
        self.wordpiece_tokenizer = WordpieceTokenizer(
            vocab=self.vocab,
            unk_token=self.unk_token,
            max_input_chars_per_word=self.max_input_chars_per_word
        )

    def tokenize(self, sequence):
        tokens = []
        if self.do_basic_tokenization:
            for token in self.basic_tokenizer.tokenize(sequence, never_split=self.never_split):
                for t in self.wordpiece_tokenizer.tokenize(token):
                    tokens.append(t)
        else:
            tokens = self.wordpiece_tokenizer.tokenize(sequence)
        return tokens


class TransformerJiebaTokenizer(TransformerVocabBasedTokenizer):

    def __init__(self, vocab_file, jieba_userdict=None, **kwargs):
        super().__init__(vocab_file, **kwargs)
        if jieba_userdict:
            jieba.load_userdict(jieba_userdict)
            logging.info('jieba load user dict: %s finished.' % jieba_userdict)

    def tokenize(self, sequence):
        tokens = []
        for t in jieba.cut(sequence):
            w = t.strip()
            if not w:
                continue
            tokens.append(w)
        return tokens



import numpy as np
import tensorflow as tf


def gelu(x):
    """ Gaussian Error Linear Unit.
    Original Implementation of the gelu activation function in Google Bert repo when initially created.
        For information: OpenAI GPT's gelu is slightly different (and gives slightly different results):
        0.5 * x * (1 + torch.tanh(math.sqrt(2 / math.pi) * (x + 0.044715 * torch.pow(x, 3))))
        Also see https://arxiv.org/abs/1606.08415
    """
    cdf = 0.5 * (1.0 + tf.math.erf(x / tf.math.sqrt(2.0)))
    return x * cdf


def gelu_new(x):
    """Gaussian Error Linear Unit.
    This is a smoother version of the RELU.
    Original paper: https://arxiv.org/abs/1606.08415
    Args:
        x: float Tensor to perform activation.
    Returns:
        `x` with the GELU activation applied.
    """
    cdf = 0.5 * (1.0 + tf.tanh((np.sqrt(2 / np.pi) * (x + 0.044715 * tf.pow(x, 3)))))
    return x * cdf


def swish(x):
    return x * tf.sigmoid(x)


ACT2FN = {
    "gelu": tf.keras.layers.Activation(gelu),
    "relu": tf.keras.activations.relu,
    "swish": tf.keras.layers.Activation(swish),
    "gelu_new": tf.keras.layers.Activation(gelu_new),
}


def choose_activation(act):
    if isinstance(act, tf.keras.layers.Activation):
        return act
    if isinstance(act, str):
        return ACT2FN[act]
    return act


def initialize(stddev=0.02):
    return tf.keras.initializers.TruncatedNormal(stddev=stddev)



import argparse
import collections
import logging
import os
import random

import tensorflow as tf

from transformers_keras.tokenizers import BertDefaultTokenizer


def create_int_feature(values):
    feature = tf.train.Feature(int64_list=tf.train.Int64List(value=list(values)))
    return feature


def create_float_feature(values):
    feature = tf.train.Feature(float_list=tf.train.FloatList(value=list(values)))
    return feature


class CustomBertGenerator(object):
    """Generate tfrecord files for training, it's different from original bert generation.
        If you need to use standard bert generation, use `bert_create_pretraining_data.py`
    """

    def __init__(self, vocab_file, **kwargs):
        super().__init__()
        self.do_lower_case = kwargs.pop('do_lower_case', True)
        self.split_on_punc = kwargs.pop('split_on_punc', True)
        self.tokenizer = BertDefaultTokenizer(vocab_file=vocab_file)
        self.rng = random.Random(12345)
        self.max_sequence_length = kwargs.pop('max_sequence_length', 512)
        self.do_whole_word_mask = kwargs.pop('do_whole_word_mask', True)
        self.masked_lm_prob = kwargs.pop('masked_lm_prob', 0.15)
        self.max_predictions_per_seq = kwargs.pop('max_predictions_per_seq', 20)
        self.vocab_words = self.tokenizer.vocab
        self.record_option = kwargs.pop('record_option', None)

    def _parse_segment_strs(self, line):
        line = line.strip('\n')
        strs = line.split('@@@')
        if len(strs) != 5:
            return '', '', -1
        label, jdid, cvid, stra, strb = strs[0], strs[1], strs[2], strs[3], strs[4]
        return stra, strb, int(label)

    def _compose_segments(self, stra, strb):
        tokens_a = self.tokenizer.tokenize(stra)
        tokens_b = self.tokenizer.tokenize(strb)
        self._truncate_seq_pair(tokens_a, tokens_b, self.max_sequence_length - 3, self.rng)
        return tokens_a, tokens_b

    def _masked_lm_process(self, instance):
        tokens = instance['original_tokens']
        outputs = self._create_masked_lm_predictions(
            tokens=tokens,
            masked_lm_prob=self.masked_lm_prob,
            max_predictions_per_seq=self.max_predictions_per_seq,
            vocab_words=self.vocab_words,
            rng=self.rng)

        # len(masked_lm_masked_tokens) == len(tokens)
        # len(masked_lm_masked_positions) == self.max_predictions_per_seq
        # len(masked_lm_origin_tokens) == self.max_predictions_per_seq
        masked_lm_masked_tokens, masked_lm_masked_positions, masked_lm_origin_tokens = outputs
        instance.update({
            'original_ids': self.tokenizer.encode(tokens),  # length == len(tokens)
            'masked_tokens': masked_lm_masked_tokens,  # lenght == len(tokens)
            # length == len(tokens)
            'masked_ids': self.tokenizer.encode(masked_lm_masked_tokens),
            'masked_lm_positions': masked_lm_masked_positions,  # length = max_predictions_per_seq
            'masked_lm_tokens': masked_lm_origin_tokens,  # length = max_predictions_per_seq
            'masked_lm_ids': self.tokenizer.encode(masked_lm_origin_tokens)
        })
        return instance

    def _create_example(self, instance):
        # instance = {
        #     'original_tokens': [], # original tokens
        #     'original_ids': [], # original ids
        #     'masked_tokens': [], # tokens contains [MASK]
        #     'masked_ids': [], # ids contains mask_id
        #     'nsp_label': 0, # nsp label
        #     'masked_lm_positions': [], # positions of masked tokens in sequence
        #     'masked_lm_tokens': [], # original tokens replaced by [MASK]
        #     'masked_lm_ids': [], # original tokens' ids replaced by [MASK]
        # }
        input_ids = instance['masked_ids']
        segment_ids = instance['segment_ids']
        original_ids = instance['original_ids']
        assert len(input_ids) == len(segment_ids)
        assert len(input_ids) == len(original_ids)

        input_mask = [1] * len(input_ids)  # masking padding positions
        while len(input_ids) < self.max_sequence_length:
            input_ids.append(self.pad_id)
            segment_ids.append(self.pad_id)
            input_mask.append(0)
            original_ids.append(0)

        assert len(input_ids) == self.max_sequence_length
        assert len(segment_ids) == self.max_sequence_length
        assert len(input_mask) == self.max_sequence_length
        assert len(original_ids) == self.max_sequence_length

        masked_lm_positions = instance['masked_lm_positions']
        masked_lm_ids = instance['masked_lm_ids']
        masked_lm_weights = [1.0] * len(masked_lm_ids)
        while len(masked_lm_positions) < self.max_predictions_per_seq:
            masked_lm_positions.append(0)
            masked_lm_ids.append(0)
            masked_lm_weights.append(0.0)

        assert len(masked_lm_positions) == self.max_predictions_per_seq
        assert len(masked_lm_ids) == self.max_predictions_per_seq
        assert len(masked_lm_weights) == self.max_predictions_per_seq

        features = collections.OrderedDict()
        features['original_ids'] = create_int_feature(instance['original_ids'])  # original ids of tokens
        features['input_ids'] = create_int_feature(input_ids)  # contains masked tokens
        features['input_mask'] = create_int_feature(input_mask)
        features['segment_ids'] = create_int_feature(segment_ids)
        features['next_sentence_labels'] = create_int_feature([instance['next_sentence_labels']])
        features['masked_lm_positions'] = create_int_feature(masked_lm_positions)
        features['masked_lm_ids'] = create_int_feature(masked_lm_ids)
        features['masked_lm_weights'] = create_float_feature(masked_lm_weights)

        example = tf.train.Example(features=tf.train.Features(feature=features))
        return example

    def generate(self, input_files, output_files):
        """Convert pretrain corpus to tfrecord files. Each line of input_files should contains two segments string.
            e.g aaaaaaa@bbbbbb
        """
        writers = []
        for output_file in output_files:
            writers.append(tf.io.TFRecordWriter(output_file, options=self.record_option))

        writer_index = 0
        total = 0
        for input_file in input_files:
            if not os.path.exists(input_file):
                logging.warning('Input file: %s does not exists.' % input_file)
                continue
            with open(input_file, mode='rt', encoding='utf8') as fin:
                while True:
                    line = fin.readline()
                    if not line:
                        break
                    stra, strb, nsp_label = self._parse_segment_strs(line)
                    if not stra or not strb or nsp_label < 0:
                        continue

                    tokens_a, tokens_b = self._compose_segments(stra, strb)
                    if not tokens_a or not tokens_b:
                        continue

                    tokens, segment_ids = ['[CLS]'], [0]
                    for t in tokens_a:
                        tokens.append(t)
                        segment_ids.append(0)
                    tokens.append('[SEP]')
                    segment_ids.append(0)
                    for t in tokens_b:
                        tokens.append(t)
                        segment_ids.append(1)
                    tokens.append('[SEP]')
                    segment_ids.append(1)

                    assert len(tokens) == len(segment_ids)

                    instance = {
                        'original_tokens': tokens,
                        'segment_ids': segment_ids,
                        'next_sentence_labels': nsp_label,
                    }

                    instance = self._masked_lm_process(instance)

                    try:
                        example = self._create_example(instance)
                        if example is not None:
                            writers[writer_index].write(example.SerializeToString())
                            total += 1
                    except Exception as e:
                        logging.warning('Create example exception: %s' % e)
                        continue

                    if total % 10000 == 0:
                        logging.info('Write %d examples.' % total)

                    # updated writer index
                    writer_index = (writer_index + 1) % len(writers)

        for writer in writers:
            writer.close()

    def _truncate_seq_pair(self, tokens_a, tokens_b, max_num_tokens, rng):
        """Truncates a pair of sequences to a maximum sequence length."""
        while True:
            total_length = len(tokens_a) + len(tokens_b)
            if total_length <= max_num_tokens:
                break

            trunc_tokens = tokens_a if len(tokens_a) > len(tokens_b) else tokens_b
            assert len(trunc_tokens) >= 1

            # We want to sometimes truncate from the front and sometimes from the
            # back to add more randomness and avoid biases.
            if rng.random() < 0.5:
                del trunc_tokens[0]
            else:
                trunc_tokens.pop()

    def _create_masked_lm_predictions(self, tokens, masked_lm_prob, max_predictions_per_seq, vocab_words, rng):
        """Creates the predictions for the masked LM objective."""

        cand_indexes = []
        for (i, token) in enumerate(tokens):
            if token == "[CLS]" or token == "[SEP]":
                continue
            # Whole Word Masking means that if we mask all of the wordpieces
            # corresponding to an original word. When a word has been split into
            # WordPieces, the first token does not have any marker and any subsequence
            # tokens are prefixed with ##. So whenever we see the ## token, we
            # append it to the previous set of word indexes.
            #
            # Note that Whole Word Masking does *not* change the training code
            # at all -- we still predict each WordPiece independently, softmaxed
            # over the entire vocabulary.
            if (self.do_whole_word_mask and len(cand_indexes) >= 1 and token.startswith("##")):
                cand_indexes[-1].append(i)
            else:
                cand_indexes.append([i])

        # cand_indexes: [[1],[2,3],[4],[5],[6,7,8]]
        rng.shuffle(cand_indexes)
        # cand_indexes: [[5],[2,3],[1],[6,7,8],[4]]

        masked_lm_masking_tokens = list(tokens)

        num_to_predict = min(max_predictions_per_seq, max(1, int(round(len(tokens) * masked_lm_prob))))

        masked_lms = []
        covered_indexes = set()
        for index_set in cand_indexes:
            if len(masked_lms) >= num_to_predict:
                break
            # If adding a whole-word mask would exceed the maximum number of
            # predictions, then just skip this candidate.
            if len(masked_lms) + len(index_set) > num_to_predict:
                continue
            is_any_index_covered = False
            for index in index_set:
                if index in covered_indexes:
                    is_any_index_covered = True
                    break
            if is_any_index_covered:
                continue
            for index in index_set:
                covered_indexes.add(index)

                masked_token = None
                # 80% of the time, replace with [MASK]
                if rng.random() < 0.8:
                    masked_token = "[MASK]"
                else:
                    # 10% of the time, keep original
                    if rng.random() < 0.5:
                        masked_token = tokens[index]
                    # 10% of the time, replace with random word
                    else:
                        masked_token = vocab_words[rng.randint(0, len(vocab_words) - 1)]

                masked_lm_masking_tokens[index] = masked_token

                masked_lms.append((index, tokens[index]))
        assert len(masked_lms) <= num_to_predict
        masked_lms = sorted(masked_lms, key=lambda x: x[0])

        masked_lm_masking_positions = []
        masked_lm_original_tokens = []
        for p in masked_lms:
            masked_lm_masking_positions.append(p[0])
            masked_lm_original_tokens.append(p[1])

        return (masked_lm_masking_tokens, masked_lm_masking_positions, masked_lm_original_tokens)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--input_file', type=str, help="""Comma splited file path""")
    parser.add_argument('--output_file', type=str, help="""Comma splited file path""")
    parser.add_argument('--vocab_file', type=str, help="""Vocab file""")
    parser.add_argument('--do_lower_case', type=bool, default=True)
    parser.add_argument('--split_on_punc', type=bool, default=True)
    parser.add_argument('--max_sequence_length', type=int, default=512)
    parser.add_argument('--max_predictions_per_seq', type=int, default=20)
    parser.add_argument('--do_whole_word_mask', type=bool, default=True)
    parser.add_argument('--masked_lm_prob', type=float, default=0.15)
    parser.add_argument('--unk_token', type=str, default='[UNK]')
    parser.add_argument('--pad_token', type=str, default='[PAD]')
    parser.add_argument('--record_option', type=str, default=None)

    args, _ = parser.parse_known_args()

    input_files = str(args.input_file).split(',')
    output_files = str(args.output_file).split(',')

    config = {
        'do_lower_case': args.do_lower_case,
        'split_on_punc': args.split_on_punc,
        'max_sequence_length': args.max_sequence_length,
        'max_predictions_per_seq': args.max_predictions_per_seq,
        'do_whole_word_mask': args.do_whole_word_mask,
        'masked_lm_prob': args.masked_lm_prob,
        'unk_token': args.unk_token,
        'pad_token': args.pad_token,
        'record_option': args.record_option
    }
    g = CustomBertGenerator(args.vocab_file, **config)
    g.generate(input_files, output_files)



import tensorflow as tf

from .layers import MultiHeadAttention
from .modeling_utils import choose_activation, initialize


class BertEmbedding(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=1,
                 max_positions=512,
                 hidden_size=768,
                 type_vocab_size=2,
                 dropout_rate=0.2,
                 stddev=0.02,
                 epsilon=1e-12,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.type_vocab_size = type_vocab_size
        self.dropout_rate = dropout_rate
        self.stddev = stddev
        self.epsilon = epsilon

        self.position_embedding = tf.keras.layers.Embedding(
            self.max_positions,
            self.hidden_size,
            embeddings_initializer=initialize(self.stddev),
            name='position_embedding'
        )
        self.token_type_embedding = tf.keras.layers.Embedding(
            self.type_vocab_size,
            self.hidden_size,
            embeddings_initializer=initialize(self.stddev),
            name='token_type_embedding'
        )
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)
        self.dropout = tf.keras.layers.Dropout(self.dropout_rate)

    def build(self, input_shape):
        self.token_embedding = self.add_weight(
            'weight',
            shape=[self.vocab_size, self.hidden_size],
            initializer=initialize(self.stddev)
        )
        super().build(input_shape)

    def call(self, inputs, mode='embedding', training=None):
        # used for masked lm
        if mode == 'linear':
            return tf.matmul(inputs, self.token_embedding, transpose_b=True)

        input_ids, token_type_ids = inputs
        seq_len = input_ids.shape[1]
        position_ids = tf.range(seq_len, dtype=tf.int32)[tf.newaxis, :]
        if token_type_ids is None:
            token_type_ids = tf.fill(input_ids.shape.as_list(), 0)

        position_embeddings = self.position_embedding(position_ids)
        token_type_embeddings = self.token_type_embedding(token_type_ids)
        token_embeddings = tf.gather(self.token_embedding, input_ids)

        embeddings = token_embeddings + token_type_embeddings + position_embeddings
        embeddings = self.layer_norm(embeddings)
        embeddings = self.dropout(embeddings, training=training)
        return embeddings

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'hidden_size': self.hidden_size,
            'type_vocab_size': self.type_vocab_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertIntermediate(tf.keras.layers.Layer):

    def __init__(self, intermediate_size=3072, activation='gelu', stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.intermediate_size = intermediate_size
        self.stddev = stddev
        self.dense = tf.keras.layers.Dense(
            self.intermediate_size, kernel_initializer=initialize(self.stddev)
        )
        self.activation = choose_activation(activation)

    def call(self, inputs, training=None):
        hidden_states = inputs
        hidden_states = self.dense(hidden_states)
        hidden_states = self.activation(hidden_states)
        return hidden_states

    def get_config(self):
        config = {
            'activation': tf.keras.activations.serialize(self.activation),
            'intermediate_size': self.intermediate_size,
            'stddev': self.stddev
        }
        base = super(BertIntermediate, self).get_config()
        return dict(list(base.items()) + list(config.items()))


class BertEncoderLayer(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 dropout_rate=0.2,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.attention = MultiHeadAttention(hidden_size=self.hidden_size, num_attention_heads=self.num_attention_heads)
        self.attn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.attn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

        self.intermediate = BertIntermediate(self.intermediate_size, self.activation, self.stddev)
        self.inter_dense = tf.keras.layers.Dense(self.hidden_size, kernel_initializer=initialize(self.stddev))
        self.inter_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.inter_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

    def call(self, inputs, training=None):
        hidden_states, mask = inputs

        attn_output, attn_weights = self.attention(inputs=(hidden_states, hidden_states, hidden_states, mask))
        attn_output = self.attn_dropout(attn_output, training=training)
        attn_output = self.attn_layer_norm(hidden_states + attn_output)

        outputs = self.intermediate(inputs=attn_output)
        outputs = self.inter_dropout(self.inter_dense(outputs), training=training)
        outputs = self.inter_layer_norm(attn_output + outputs)

        return outputs, attn_weights

    def get_config(self):
        config = {
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers=6,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 dropout_rate=0.2,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        self.num_layers = num_layers
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.encoder_layers = [
            BertEncoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                intermediate_size=intermediate_size,
                activation=activation,
                dropout_rate=dropout_rate,
                epsilon=epsilon,
                stddev=stddev,
                name='BertEncoderLayer{}'.format(i)
            ) for i in range(self.num_layers)
        ]

    def call(self, inputs, training=None):
        hidden_states, attention_mask = inputs
        all_hidden_states = []
        all_attention_scores = []
        for _, encoder in enumerate(self.encoder_layers):
            hidden_states, attention_score = encoder(inputs=(hidden_states, attention_mask))
            all_hidden_states.append(hidden_states)
            all_attention_scores.append(attention_score)

        return hidden_states, all_hidden_states, all_attention_scores

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertPooler(tf.keras.layers.Layer):

    def __init__(self, hidden_size=768, stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.hidden_size = hidden_size
        self.stddev = stddev
        self.dense = tf.keras.layers.Dense(
            self.hidden_size, kernel_initializer=initialize(self.stddev), activation='tanh')

    def call(self, inputs, training=None):
        hidden_states = inputs
        # pool the first token: [CLS]
        outputs = self.dense(hidden_states[:, 0])
        return outputs

    def get_config(self):
        config = {
            'hidden_size': self.hidden_size,
            'stddev': 0.02,
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertModel(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=1,
                 max_positions=512,
                 hidden_size=768,
                 type_vocab_size=2,
                 num_layers=6,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 dropout_rate=0.2,
                 stddev=0.02,
                 epsilon=1e-12,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.type_vocab_size = type_vocab_size
        self.num_layers = num_layers
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.stddev = stddev
        self.epsilon = epsilon

        self.bert_embedding = BertEmbedding(
            vocab_size=self.vocab_size,
            max_positions=self.max_positions,
            hidden_size=self.hidden_size,
            type_vocab_size=self.type_vocab_size,
            dropout_rate=self.dropout_rate,
            stddev=self.stddev,
            epsilon=self.epsilon,
            **kwargs)

        self.bert_encoder = BertEncoder(
            num_layers=self.num_layers,
            hidden_size=self.hidden_size,
            num_attention_heads=self.num_attention_heads,
            intermediate_size=self.intermediate_size,
            activation=self.activation,
            dropout_rate=self.dropout_rate,
            epsilon=self.epsilon,
            stddev=self.stddev,
            **kwargs)

        self.bert_pooler = BertPooler(hidden_size=self.hidden_size, stddev=self.stddev, **kwargs)

    def call(self, inputs, training=None):
        input_ids, token_type_ids, mask = inputs
        mask = mask[:, tf.newaxis, tf.newaxis, :]  # (batch_size, seq_len) -> (batch_size, 1, 1, seq_len)
        embedding = self.bert_embedding(inputs=(input_ids, token_type_ids), mode='embedding')
        output, all_hidden_states, all_attention_scores = self.bert_encoder(inputs=(embedding, mask))
        pooled_output = self.bert_pooler(output)
        return output, pooled_output, all_hidden_states, all_attention_scores

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'type_vocab_size': self.type_vocab_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertMLMHead(tf.keras.layers.Layer):
    """Masked language model for BERT pre-training."""

    def __init__(self,
                 embedding,
                 vocab_size=-1,
                 hidden_size=768,
                 activation='gelu',
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.embedding = embedding
        self.hidden_size = hidden_size
        self.activation = choose_activation(activation)
        self.epsilon = epsilon
        self.stddev = stddev

        self.dense = tf.keras.layers.Dense(self.hidden_size, kernel_initializer=initialize(stddev))
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

    def build(self, input_shape):
        self.bias = self.add_weight(shape=(self.vocab_size,), initializer='zeros', trainable=True, name='bias')
        super().build(input_shape)

    def call(self, inputs, training=None):
        hidden_states = inputs
        hidden_states = self.dense(hidden_states)
        hidden_states = self.activation(hidden_states)
        hidden_states = self.layer_norm(hidden_states)
        hidden_states = self.embedding(inputs=hidden_states, mode='linear')
        hidden_states = hidden_states + self.bias
        return hidden_states

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'hidden_size': self.hidden_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'epsilon': self.epsilon,
            'stddev': self.stddev
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class BertNSPHead(tf.keras.layers.Layer):
    """Next sentence prediction for BERT pre-training."""

    def __init__(self, stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.stddev = stddev
        self.classifier = tf.keras.layers.Dense(2, kernel_initializer=initialize(self.stddev))

    def call(self, inputs, training=None):
        pooled_output = inputs
        relation = self.classifier(pooled_output)
        return relation

    def get_config(self):
        config = {
            'stddev': self.stddev
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class Bert4PreTraining(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=-1,
                 max_positions=512,
                 hidden_size=768,
                 type_vocab_size=2,
                 num_layers=6,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 dropout_rate=0.2,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.type_vocab_size = type_vocab_size
        self.num_layers = num_layers
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.bert = BertModel(
            vocab_size=self.vocab_size,
            max_positions=self.max_positions,
            hidden_size=self.hidden_size,
            type_vocab_size=self.type_vocab_size,
            num_layers=self.num_layers,
            num_attention_heads=self.num_attention_heads,
            intermediate_size=self.intermediate_size,
            activation=self.activation,
            dropout_rate=self.dropout_rate,
            stddev=self.stddev,
            epsilon=self.epsilon)
        self.mlm = BertMLMHead(
            vocab_size=self.vocab_size,
            embedding=self.bert.bert_embedding,
            hidden_size=self.hidden_size,
            activation=self.activation,
            epsilon=self.epsilon,
            stddev=self.stddev)
        self.nsp = BertNSPHead(stddev=stddev, name='NSP')

    def call(self, inputs, training=None):
        input_ids, token_type_ids, mask = inputs
        mask = tf.cast(mask, dtype=tf.float32)
        sequence_output, pooled_output, _, all_attention_scores = self.bert(inputs=(input_ids, token_type_ids, mask))
        prediction_scores = self.mlm(sequence_output)
        relation_scores = self.nsp(pooled_output)
        return prediction_scores, relation_scores, all_attention_scores

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'type_vocab_size': self.type_vocab_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))



from .abstract_dataset_builder import AbstractDatasetBuilder
from .bert_dataset_builder import BertTFRecordDatasetBuilder
from .transformer_dataset_builder import TransformerDatasetBuilder
from .transformer_dataset_builder import TransformerTFRecordDatasetBuilder
from .transformer_dataset_builder import TransformerTextFileDatasetBuilder



import unittest

import tensorflow as tf

from .bert_tfrecord_custom_generator import CustomBertGenerator


class CustomBertGeneratorTest(unittest.TestCase):

    def testGenerateTFRecordFiles(self):
        config = {
            'do_lower_case': True,
            'split_on_punc': True,
            'max_sequence_length': 128,
            'max_predictions_per_seq': 20,
            'do_whole_word_mask': True,
            'masked_lm_prob': 0.15,
            'unk_token': '[UNK]',
            'pad_token': '[PAD]',
            'record_option': 'GZIP'
        }
        vocab_file = 'testdata/bert_vocab.txt'
        input_files = [
            'testdata/bert_custom_corpus.txt'
        ]
        output_files = [
            'testdata/bert_custom_pretrain.tfrecord'
        ]
        g = CustomBertGenerator(vocab_file, **config)
        g.generate(input_files, output_files)

    def testReadTFRecordFile(self):
        f = 'testdata/bert_custom_pretrain.tfrecord'
        dataset = tf.data.TFRecordDataset(f, compression_type='GZIP')
        for d in dataset.take(1):
            print(d)

        name_to_features = {
            'original_ids': tf.io.FixedLenFeature([128], tf.int64),
            'input_ids': tf.io.FixedLenFeature([128], tf.int64),
            'input_mask': tf.io.FixedLenFeature([128], tf.int64),
            'segment_ids': tf.io.FixedLenFeature([128], tf.int64),
            'next_sentence_labels': tf.io.FixedLenFeature([1], tf.int64),
            'masked_lm_positions': tf.io.FixedLenFeature([20], tf.int64),
            'masked_lm_weights': tf.io.FixedLenFeature([20], tf.float32),
            'masked_lm_ids': tf.io.FixedLenFeature([20], tf.int64),
        }

        def _parse_example(record):
            example = tf.io.parse_single_example(record, name_to_features)
            return example

        dataset = dataset.map(lambda x: _parse_example(x))
        dataset = dataset.batch(4)
        for d in dataset.take(1):
            print(d)


if __name__ == "__main__":
    unittest.main()



import numpy as np
import tensorflow as tf

from .layers import DecoderLayer, EncoderLayer


class PositionalEncoding(tf.keras.layers.Layer):

    def __init__(self, max_positions=512, embedding_size=512, **kwargs):
        super().__init__(**kwargs)
        self.max_positions = max_positions
        self.embedding_size = embedding_size

    def build(self, input_shape):

        def _initializer(shape, dtype=tf.float32):
            pos = np.arange(self.max_positions)[:, tf.newaxis]
            d = np.arange(self.embedding_size)[tf.newaxis, :]
            rads = 1 / np.power(10000, (2 * (d // 2)) / np.float32(self.embedding_size))
            rads = pos * rads

            rads[:, 0::2] = np.sin(rads[:, 0::2])
            rads[:, 1::2] = np.cos(rads[:, 1::2])

            rads = tf.cast(rads, dtype=dtype)
            rads = tf.reshape(rads, shape=shape)
            return rads

        self.position_embedding = self.add_weight(
            name='position_embedding',
            shape=(self.max_positions, self.embedding_size),
            dtype=tf.float32,
            initializer=_initializer,
            trainable=False)

        super().build(input_shape)

    def call(self, inputs, training=None):
        token_ids = inputs
        pos_embedding = self.position_embedding[tf.newaxis, :]
        embedding = pos_embedding[:, :tf.shape(token_ids)[1], :]
        return embedding

    def get_config(self):
        config = {
            'max_positions': self.max_positions,
            'embedding_size': self.embedding_size
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class TransformerEmbedding(tf.keras.layers.Layer):

    def __init__(self, vocab_size=1, max_positions=512, embedding_size=512, dropout_rate=0.2, **kwargs):
        super(TransformerEmbedding, self).__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.embedding_size = embedding_size
        self.dropout_rate = dropout_rate
        self.dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.token_embedding = tf.keras.layers.Embedding(self.vocab_size, self.embedding_size)
        self.positional_encoding = PositionalEncoding(self.max_positions, self.embedding_size)

    def call(self, inputs, training=None):
        token_ids = inputs
        token_embeddings = self.token_embedding(token_ids)
        position_embeddings = self.positional_encoding(token_ids)
        embedding = token_embeddings + position_embeddings
        embedding = self.dropout(embedding, training=training)
        return embedding

    def get_config(self):
        conf = {
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'embedding_size': self.embedding_size,
            'dropout_rate': self.dropout_rate
        }
        p = super(TransformerEmbedding, self).get_config()
        return dict(list(p.items()) + list(conf.items()))


class TransformerEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=-1,
                 max_positions=512,
                 hidden_size=512,
                 num_layers=6,
                 num_attention_heads=8,
                 ffn_size=2048,
                 dropout_rate=0.2,
                 epsilon=1e-6,
                 **kwargs):
        super(TransformerEncoder, self).__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.num_attention_heads = num_attention_heads
        self.ffn_size = ffn_size
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.embedding = TransformerEmbedding(vocab_size, max_positions, hidden_size, dropout_rate)
        self.encoders = [
            EncoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                ffn_size=ffn_size,
                dropout_rate=dropout_rate,
                epsilon=epsilon,
                name='EncoderLayer{}'.format(i)
            ) for i in range(self.num_layers)
        ]

    def call(self, inputs, training=None):
        token_ids, mask = inputs
        embeddings = self.embedding(inputs=token_ids)

        attn_weights = []
        outputs = embeddings
        for i in range(self.num_layers):
            encoder = self.encoders[i]
            outputs, weights = encoder(inputs=(outputs, outputs, outputs, mask))
            attn_weights.append(weights)

        return outputs, attn_weights

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'ffn_size': self.ffn_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon
        }
        p = super().get_config()
        return dict(list(p.items()) + list(config.items()))


class TransformerDecoder(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=-1,
                 max_positions=512,
                 hidden_size=512,
                 num_layers=6,
                 num_attention_heads=8,
                 ffn_size=2048,
                 dropout_rate=0.2,
                 epsilon=1e-6,
                 **kwargs):
        super(TransformerDecoder, self).__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.num_attention_heads = num_attention_heads
        self.ffn_size = ffn_size
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.embedding = TransformerEmbedding(vocab_size, max_positions, hidden_size, dropout_rate)
        self.decoders = [
            DecoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                ffn_size=ffn_size,
                dropout_rate=dropout_rate,
                epsilon=epsilon,
                name='DecoderLayer{}'.format(i)
            ) for i in range(self.num_layers)
        ]

    def call(self, inputs, training=None):
        token_ids, enc_outputs, look_ahead_mask, padding_mask = inputs
        embeddings = self.embedding(token_ids)

        self_attn_weights, context_attn_weights = [], []
        outputs = embeddings

        for i in range(self.num_layers):
            decoder = self.decoders[i]
            outputs, self_attn, context_attn = decoder(inputs=(outputs, enc_outputs, look_ahead_mask, padding_mask))
            self_attn_weights.append(self_attn)
            context_attn_weights.append(context_attn)

        return outputs, self_attn_weights, context_attn_weights

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'ffn_size': self.ffn_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon
        }
        p = super(TransformerDecoder, self).get_config()
        return dict(list(p.items()) + list(config.items()))


class Transformer(tf.keras.layers.Layer):

    def __init__(self,
                 src_vocab_size=-1,
                 tgt_vocab_size=-1,
                 max_positions=512,
                 hidden_size=512,
                 num_encoder_layers=6,
                 num_decoder_layers=6,
                 num_attention_heads=8,
                 ffn_size=2048,
                 dropout_rate=0.2,
                 epsilon=1e-6,
                 **kwargs):
        super(Transformer, self).__init__(**kwargs)
        assert src_vocab_size > 0, "src_vocab_size must greater than 0."
        assert tgt_vocab_size > 0, "tgt_vocab_size must greater than 0."
        self.src_vocab_size = src_vocab_size
        self.tgt_vocab_size = tgt_vocab_size
        self.max_positions = max_positions
        self.hidden_size = hidden_size
        self.num_encoder_layers = num_encoder_layers
        self.num_decoder_layers = num_decoder_layers
        self.num_attention_heads = num_attention_heads
        self.ffn_size = ffn_size
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.encoder = TransformerEncoder(
            src_vocab_size, max_positions, hidden_size,
            num_layers=num_encoder_layers, dropout_rate=dropout_rate, epsilon=epsilon)
        self.decoder = TransformerDecoder(
            tgt_vocab_size, max_positions, hidden_size,
            num_layers=num_encoder_layers, dropout_rate=dropout_rate, epsilon=epsilon)
        self.dense = tf.keras.layers.Dense(tgt_vocab_size)

    def call(self, inputs, training=None):
        x_ids, y_ids = inputs

        def _create_padding_mask(x):
            mask = tf.cast(tf.equal(0, x), dtype=tf.float32)
            mask = mask[:, tf.newaxis, tf.newaxis, :]
            return mask

        def _create_look_ahead_mask(size):
            mask = 1 - tf.linalg.band_part(tf.ones((size, size)), -1, 0)
            return mask

        def _create_masks(x, y):
            _enc_padding_mask = _create_padding_mask(x)
            _dec_padding_mask = _create_padding_mask(x)
            _look_ahead_mask = _create_look_ahead_mask(tf.shape(y)[1])
            _target_padding_mask = _create_padding_mask(y)
            combined = tf.maximum(_look_ahead_mask, _target_padding_mask)
            return _enc_padding_mask, combined, _dec_padding_mask

        enc_padding_mask, dec_look_ahead_mask, dec_padding_mask = _create_masks(x_ids, y_ids)

        enc_outputs, enc_attns = self.encoder(inputs=(x_ids, enc_padding_mask))

        dec_outputs, dec_attns_0, dec_attns_1 = self.decoder(
            inputs=(y_ids, enc_outputs, dec_look_ahead_mask, dec_padding_mask))

        logits = self.dense(dec_outputs)
        return logits, enc_attns, dec_attns_0, dec_attns_1

    def get_config(self):
        config = {
            "src_vocab_size": self.src_vocab_size,
            "tgt_vocab_size": self.tgt_vocab_size,
            "max_positions": self.max_positions,
            "hidden_size": self.hidden_size,
            "num_encoder_layers": self.num_encoder_layers,
            "num_decoder_layers": self.num_decoder_layers,
            "num_attention_heads": self.num_attention_heads,
            "ffn_size": self.ffn_size,
            "dropout_rate": self.dropout_rate,
            "epsilon": self.epsilon,
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))



import tensorflow as tf

from transformers_keras.tokenizers import TransformerAbstractTokenizer

from .abstract_dataset_builder import AbstractDatasetBuilder


class TransformerDatasetBuilder(AbstractDatasetBuilder):

    def __init__(self, pad_id=0, **kwargs):
        super().__init__(**kwargs)
        self.pad_id = pad_id

    def _repeat(self, dataset, mode='train'):
        repeat = 0
        if 'train' == mode:
            repeat = self.train_repeat_count
        elif 'valid' == mode:
            repeat = self.valid_repeat_count
        elif 'predict' == mode:
            repeat = self.predict_repeat_count
        else:
            raise ValueError('Invalid mode: %s, must be in [train, valid, predict]' % mode)
        if not repeat or repeat <= 0:
            return dataset
        dataset = dataset.repeat(repeat)
        return dataset

    def _shuffle(self, dataset, mode='train'):
        if 'predict' == mode:
            return dataset
        elif 'train' == mode:
            dataset = dataset.shuffle(
                buffer_size=self.train_shuffle_buffer_size,
                seed=self.train_shuffle_seed,
                reshuffle_each_iteration=self.train_reshuffle_each_iteration)
            return dataset
        elif 'valid' == mode:
            dataset = dataset.shuffle(
                buffer_size=self.valid_shuffle_buffer_size,
                seed=self.valid_shuffle_seed,
                reshuffle_each_iteration=self.valid_reshuffle_each_iteration)
            return dataset
        else:
            raise ValueError('Invalid mode: %s, must be in [train, valid, predict]' % mode)

    def build_train_dataset(self, train_files, **kwargs):
        raise NotImplementedError()

    def build_valid_dataset(self, valid_files, **kwargs):
        raise NotImplementedError()

    def build_predict_dataset(self, valid_files, **kwargs):
        raise NotImplementedError()


class TransformerTFRecordDatasetBuilder(TransformerDatasetBuilder):
    """Build dataset from tfrecord files."""

    # TODO: Shift tgt sequence when generating tfrecord files

    def __init__(self, src_max_len=512, tgt_max_len=512, record_option='GZIP', **kwargs):
        super().__init__(**kwargs)
        self.src_max_len = src_max_len
        self.tgt_max_len = tgt_max_len
        self.compression_type = record_option

    def _build_dataset_from_tfrecord_files(self, files, skip=0):
        dataset = tf.data.Dataset.from_tensor_slices(files)
        dataset = dataset.interleave(
            lambda x: tf.data.TFRecordDataset(x, compression_type=self.compression_type).skip(skip),
            cycle_length=len(files)
        )
        return dataset

    def _parse_example_fn(self, x):
        name_to_features = {
            'src_ids': tf.io.FixedLenFeature([self.src_max_len], tf.int64),
            'tgt_ids': tf.io.FixedLenFeature([self.tgt_max_len], tf.int64)
        }
        example = tf.io.parse_single_example(x, name_to_features)
        features = example['src_ids']
        labels = example['tgt_ids']
        return (features, labels)

    def build_train_dataset(self, train_files, **kwargs):
        dataset = self._build_dataset_from_tfrecord_files(train_files, skip=self.train_skip_count)
        dataset = self._repeat(dataset, mode='train')
        dataset = self._shuffle(dataset, mode='train')
        dataset = dataset.map(lambda x: self._parse_example_fn(x), num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.filter(lambda x, y: tf.logical_and(tf.size(x) > 0, tf.size(y) > 0))
        dataset = dataset.batch(self.train_batch_size, drop_remainder=self.train_drop_remainder)
        # should shift tgt sequence when generating tfrecord files
        dataset = dataset.map(lambda x, y: ((x, y[:, :-1]), y[:, 1:]), num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.prefetch(self.prefetch_size)
        return dataset

    def build_valid_dataset(self, valid_files, **kwargs):
        dataset = self._build_dataset_from_tfrecord_files(valid_files, skip=self.valid_skip_count)
        dataset = self._repeat(dataset, mode='valid')
        dataset = self._shuffle(dataset, mode='valid')
        dataset = dataset.map(lambda x: self._parse_example_fn(x), num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.filter(lambda x, y: tf.logical_and(tf.size(x) > 0, tf.size(y) > 0))
        dataset = dataset.batch(self.train_batch_size, drop_remainder=self.valid_drop_remainder)
        dataset = dataset.map(lambda x, y: ((x, y[:, :-1]), y[:, 1:]), num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.prefetch(self.prefetch_size)
        return dataset

    def build_predict_dataset(self, predict_files, **kwargs):
        # prediction process is different from training and evaluation
        raise NotImplementedError()


class TransformerTextFileDatasetBuilder(TransformerDatasetBuilder):

    def __init__(self,
                 src_tokenizer: TransformerAbstractTokenizer,
                 tgt_tokenizer: TransformerAbstractTokenizer,
                 pad_id=0,
                 **kwargs):
        super().__init__(pad_id=pad_id, **kwargs)
        self.src_tokenizer = src_tokenizer
        self.tgt_tokenizer = tgt_tokenizer
        self.sep = kwargs.get('sep', '@@@')

    def _build_dataset_from_text_files(self, files, mode='train'):
        assert all(isinstance(x, tuple) for x in files), 'Invalid input files format.'
        src_files, tgt_files = [x[0] for x in files], [x[1] for x in files]
        src_dataset = tf.data.Dataset.from_tensor_slices(src_files)
        tgt_dataset = tf.data.Dataset.from_tensor_slices(tgt_files)

        skip = 0
        if 'train' == mode:
            skip = self.train_skip_count
        elif 'valid' == mode:
            skip = self.valid_skip_count
        elif 'predict' == mode:
            skip = self.predict_skip_count
        else:
            skip = 0
        if not skip or skip <= 0:
            skip = 0

        src_dataset = src_dataset.flat_map(lambda x: tf.data.TextLineDataset(x).skip(skip))
        tgt_dataset = tgt_dataset.flat_map(lambda x: tf.data.TextLineDataset(x).skip(skip))
        dataset = tf.data.Dataset.zip((src_dataset, tgt_dataset))
        return dataset

    def _parse_example(self, dataset):

        def _parse(x, y):
            src = x.numpy().decode('utf8')
            tgt = y.numpy().decode('utf8')
            src = [self.src_tokenizer.sos_id] + self.src_tokenizer.encode(src) + [self.src_tokenizer.eos_id]
            tgt = [self.tgt_tokenizer.sos_id] + self.tgt_tokenizer.encode(tgt) + [self.tgt_tokenizer.eos_id]
            return src, tgt

        def _tf_parse_fn(x, y):
            src, tgt = tf.py_function(_parse, [x, y], [tf.int64, tf.int64])
            src.set_shape([None])
            tgt.set_shape([None])
            return src, tgt

        dataset = dataset.map(lambda x, y: _tf_parse_fn(x, y), num_parallel_calls=self.num_parallel_calls)
        return dataset

    def build_train_dataset(self, train_files, **kwargs):
        dataset = self._build_dataset_from_text_files(train_files, mode='train')
        dataset = self._repeat(dataset, mode='train')
        dataset = self._shuffle(dataset, mode='train')
        dataset = self._parse_example(dataset)
        dataset = dataset.filter(lambda x, y: tf.logical_and(tf.size(x) > 0, tf.size(y) > 0))

        pad_id = tf.constant(self.pad_id, dtype=tf.int64)
        dataset = dataset.padded_batch(
            batch_size=self.train_batch_size,
            padded_shapes=([None], [None]),
            padding_values=(pad_id, pad_id),
            drop_remainder=self.train_drop_remainder,
        ).prefetch(self.prefetch_size)
        # x-> encoder input, y->decoder input, z->label
        dataset = dataset.map(lambda x, y: ((x, y[:, :-1]), y[:, 1:]))
        return dataset

    def build_valid_dataset(self, valid_files, **kwargs):
        dataset = self._build_dataset_from_text_files(valid_files, mode='valid')
        dataset = self._repeat(dataset, mode='valid')
        dataset = self._shuffle(dataset, mode='valid')
        dataset = self._parse_example(dataset)
        dataset = dataset.filter(lambda x, y: tf.logical_and(tf.size(x) > 0, tf.size(y) > 0))

        pad_id = tf.constant(self.pad_id, dtype=tf.int64)
        dataset = dataset.padded_batch(
            batch_size=self.valid_batch_size,
            padded_shapes=([None], [None]),
            padding_values=(pad_id, pad_id),
            drop_remainder=self.valid_drop_remainder,
        ).prefetch(self.prefetch_size)
        # x-> encoder input, y->decoder input, z->label
        dataset = dataset.map(lambda x, y: ((x, y[:, :-1]), y[:, 1:]))
        return dataset

    def build_predict_dataset(self, predict_files, **kwargs):
        # prediction process is different from training and evaluation
        raise NotImplementedError()



import tensorflow as tf
import numpy as np

from .modeling_albert import AlbertEmbedding, AlbertEncoderLayer, AlbertEncoderGroup, AlbertEncoder
from .modeling_albert import AlbertMLMHead, AlbertSOPHead, AlbertModel, Albert4PreTraining


class ModelingAlbertTest(tf.test.TestCase):

    def testAlbertEmbedding(self):
        embedding = AlbertEmbedding(vocab_size=100, embedding_size=128)
        inputs = (
            tf.constant([[0, 2, 3, 4, 5, 1]]),  # input_ids
            tf.constant([[0, 0, 0, 1, 1, 1]]),  # token_type_ids
        )
        outputs = embedding(inputs, training=True)
        self.assertAllEqual([1, 6, 128], outputs.shape)

    def testAlbertEncoderLayer(self):
        encoder = AlbertEncoderLayer()
        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        outputs, attn_weights = encoder(inputs=(hidden_states, mask[:, tf.newaxis, tf.newaxis, :]))
        self.assertAllEqual([2, 16, 768], outputs.shape)
        self.assertAllEqual([2, 8, 16, 16], attn_weights.shape)

    def testAlbertEncoderGroup(self):

        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        mask = mask[:, tf.newaxis, tf.newaxis, :]

        def _run_albert_encoder_group(num_layers_each_group):
            group = AlbertEncoderGroup(num_layers_each_group=num_layers_each_group)
            outputs, group_states, group_attn_weights = group(inputs=(hidden_states, mask))
            self.assertAllEqual([2, 16, 768], outputs.shape)

            self.assertEqual(num_layers_each_group, len(group_states))
            for state in group_states:
                self.assertAllEqual([2, 16, 768], state.shape)

            self.assertEqual(num_layers_each_group, len(group_attn_weights))
            for attention in group_attn_weights:
                self.assertAllEqual([2, 8, 16, 16], attention.shape)

        for layer in [1, 2, 3, 4]:
            _run_albert_encoder_group(layer)

    def testAlbertEncoder(self):
        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        mask = mask[:, tf.newaxis, tf.newaxis, :]

        encoder = AlbertEncoder(num_layers=4, num_groups=1, num_layers_each_group=1, hidden_size=768)

        outputs, all_states, all_attn_weights = encoder(inputs=(hidden_states, mask))
        self.assertAllEqual([2, 16, 768], outputs.shape)

        self.assertEqual(4, len(all_states))
        for state in all_states:
            self.assertAllEqual([2, 16, 768], state.shape)

        self.assertEqual(4, len(all_attn_weights))
        for attention in all_attn_weights:
            self.assertAllEqual([2, 8, 16, 16], attention.shape)

    def testAlbertModel(self):
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]], dtype=np.float32)  # input_mask

        model = AlbertModel(vocab_size=100, hidden_size=768,
                            num_layers=4, num_groups=1, num_layers_each_group=1)
        outputs, pooled_outputs, all_states, all_attn_weights = model(inputs=(input_ids, token_type_ids, input_mask))
        self.assertAllEqual([2, 16, 768], outputs.shape)
        self.assertAllEqual([2, 768], pooled_outputs.shape)

        self.assertEqual(4, len(all_states))
        for state in all_states:
            self.assertAllEqual([2, 16, 768], state.shape)

        self.assertEqual(4, len(all_attn_weights))
        for attention in all_attn_weights:
            self.assertAllEqual([2, 8, 16, 16], attention.shape)

    def testAlbertMLMHead(self):
        embedding = AlbertEmbedding(vocab_size=100)
        mlm = AlbertMLMHead(vocab_size=100, embedding=embedding)
        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = mlm(inputs)
        self.assertAllEqual([2, 16, 100], outputs.shape)

    def testAlbertSOPHead(self):
        sop = AlbertSOPHead()
        inputs = tf.random.uniform(shape=(2, 768))
        outputs = sop(inputs)
        self.assertAllEqual([2, 2], outputs.shape)

    def testAlbert4PreTraining(self):
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]], dtype=np.float32)  # input_mask

        model = Albert4PreTraining(vocab_size=100, num_layers=4, num_groups=1, num_layers_each_group=1)
        outputs, pooled_outputs, all_states, all_attn_weights = model(inputs=(input_ids, token_type_ids, input_mask))
        self.assertAllEqual([2, 16, 100], outputs.shape)
        self.assertAllEqual([2, 2], pooled_outputs.shape)

        self.assertEqual(4, len(all_states))
        for state in all_states:
            self.assertAllEqual([2, 16, 768], state.shape)

        self.assertEqual(4, len(all_attn_weights))
        for attention in all_attn_weights:
            self.assertAllEqual([2, 8, 16, 16], attention.shape)


if __name__ == "__main__":
    tf.test.main()



import unittest

from .bert_tokenizer import BertDefaultTokenizer


class BertTokenizerTest(unittest.TestCase):

    def testBertDefaultTokenizer(self):
        tokenizer = BertDefaultTokenizer(
            'testdata/vocab_src.txt',
            unk_token='<unk>', sos_token='<s>', eos_token='</s>', pad_token='<pad>')
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(1, tokenizer.unk_id)
        self.assertEqual(2, tokenizer.sos_id)
        self.assertEqual(3, tokenizer.eos_id)
        self.assertEqual(4, tokenizer.cls_id)
        self.assertEqual(5, tokenizer.sep_id)
        self.assertEqual(6, tokenizer.mask_id)
        self.assertEqual('<pad>', tokenizer.pad_token)
        self.assertEqual('<unk>', tokenizer.unk_token)
        self.assertEqual('<s>', tokenizer.sos_token)
        self.assertEqual('</s>', tokenizer.eos_token)
        self.assertEqual('[CLS]', tokenizer.cls_token)
        self.assertEqual('[SEP]', tokenizer.sep_token)
        self.assertEqual('[MASK]', tokenizer.mask_token)

        seq = '我 住在 上海'
        self.assertEqual(['我', '住', '在', '上', '海'], tokenizer.tokenize(seq))
        self.assertEqual([7, 8, 9, 10, 11], tokenizer.encode(seq))
        self.assertEqual(12, tokenizer.vocab_size)


if __name__ == "__main__":
    unittest.main()



import abc

import tensorflow as tf


class AbstractDatasetBuilder(abc.ABC):

    def __init__(self,
                 train_batch_size=32,
                 train_skip_count=0,
                 train_repeat_count=1,
                 train_shuffle_buffer_size=1000000,
                 train_shuffle_seed=None,
                 train_reshuffle_each_iteration=True,
                 train_drop_remainder=False,
                 valid_batch_size=32,
                 valid_skip_count=0,
                 valid_repeat_count=1,
                 valid_shuffle_buffer_size=100000,
                 valid_shuffle_seed=None,
                 valid_reshuffle_each_iteration=True,
                 valid_drop_remainder=False,
                 predict_batch_size=1,
                 predict_skip_count=0,
                 predict_repeat_count=None,
                 predict_drop_remainder=False,
                 num_parallel_calls=tf.data.experimental.AUTOTUNE,
                 prefetch_size=tf.data.experimental.AUTOTUNE,
                 **kwargs):
        super().__init__()
        self.num_parallel_calls = num_parallel_calls
        self.prefetch_size = prefetch_size

        self.train_batch_size = train_batch_size
        self.train_skip_count = train_skip_count
        self.train_repeat_count = train_repeat_count
        self.train_shuffle_buffer_size = train_shuffle_buffer_size
        self.train_shuffle_seed = train_shuffle_seed
        self.train_reshuffle_each_iteration = train_reshuffle_each_iteration
        self.train_drop_remainder = train_drop_remainder

        self.valid_batch_size = valid_batch_size
        self.valid_skip_count = valid_skip_count
        self.valid_repeat_count = valid_repeat_count
        self.valid_shuffle_buffer_size = valid_shuffle_buffer_size
        self.valid_shuffle_seed = valid_shuffle_seed
        self.valid_reshuffle_each_iteration = valid_reshuffle_each_iteration
        self.valid_drop_remainder = valid_drop_remainder

        self.predict_batch_size = predict_batch_size
        self.predict_skip_count = predict_skip_count
        self.predict_repeat_count = predict_repeat_count
        self.predict_drop_remainder = predict_drop_remainder

    def build_train_dataset(self, train_files, **kwargs):
        raise NotImplementedError()

    def build_valid_dataset(self, valid_files, **kwargs):
        raise NotImplementedError()

    def build_predict_dataset(self, valid_files, **kwargs):
        raise NotImplementedError()



# coding=utf-8
# Copyright 2018 The Google AI Language Team Authors and The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import collections
import logging
import os
import unicodedata


def load_vocab(vocab_file):
    """Loads a vocabulary file into a dictionary."""
    vocab = collections.OrderedDict()
    with open(vocab_file, "r", encoding="utf-8") as reader:
        tokens = reader.readlines()
    for index, token in enumerate(tokens):
        token = token.rstrip("\n")
        vocab[token] = index
    return vocab


def whitespace_tokenize(text):
    """Runs basic whitespace cleaning and splitting on a piece of text."""
    text = text.strip()
    if not text:
        return []
    tokens = text.split()
    return tokens


class BasicTokenizer(object):
    """Runs basic tokenization (punctuation splitting, lower casing, etc.)."""

    def __init__(self, do_lower_case=True, never_split=None, tokenize_chinese_chars=True):
        """ Constructs a BasicTokenizer.

        Args:
            **do_lower_case**: Whether to lower case the input.
            **never_split**: (`optional`) list of str
                Kept for backward compatibility purposes.
                Now implemented directly at the base class level (see :func:`PreTrainedTokenizer.tokenize`)
                List of token not to split.
            **tokenize_chinese_chars**: (`optional`) boolean (default True)
                Whether to tokenize Chinese characters.
                This should likely be deactivated for Japanese:
                see: https://github.com/huggingface/pytorch-pretrained-BERT/issues/328
        """
        if never_split is None:
            never_split = []
        self.do_lower_case = do_lower_case
        self.never_split = never_split
        self.tokenize_chinese_chars = tokenize_chinese_chars

    def tokenize(self, text, never_split=None):
        """ Basic Tokenization of a piece of text.
            Split on "white spaces" only, for sub-word tokenization, see WordPieceTokenizer.

        Args:
            **never_split**: (`optional`) list of str
                Kept for backward compatibility purposes.
                Now implemented directly at the base class level (see :func:`PreTrainedTokenizer.tokenize`)
                List of token not to split.
        """
        never_split = self.never_split + (never_split if never_split is not None else [])
        text = self._clean_text(text)
        # This was added on November 1st, 2018 for the multilingual and Chinese
        # models. This is also applied to the English models now, but it doesn't
        # matter since the English models were not trained on any Chinese data
        # and generally don't have any Chinese data in them (there are Chinese
        # characters in the vocabulary because Wikipedia does have some Chinese
        # words in the English Wikipedia.).
        if self.tokenize_chinese_chars:
            text = self._tokenize_chinese_chars(text)
        orig_tokens = whitespace_tokenize(text)
        split_tokens = []
        for token in orig_tokens:
            if self.do_lower_case and token not in never_split:
                token = token.lower()
                token = self._run_strip_accents(token)
            split_tokens.extend(self._run_split_on_punc(token, never_split))

        output_tokens = whitespace_tokenize(" ".join(split_tokens))
        return output_tokens

    def _run_strip_accents(self, text):
        """Strips accents from a piece of text."""
        text = unicodedata.normalize("NFD", text)
        output = []
        for char in text:
            cat = unicodedata.category(char)
            if cat == "Mn":
                continue
            output.append(char)
        return "".join(output)

    def _run_split_on_punc(self, text, never_split=None):
        """Splits punctuation on a piece of text."""
        if never_split is not None and text in never_split:
            return [text]
        chars = list(text)
        i = 0
        start_new_word = True
        output = []
        while i < len(chars):
            char = chars[i]
            if _is_punctuation(char):
                output.append([char])
                start_new_word = True
            else:
                if start_new_word:
                    output.append([])
                start_new_word = False
                output[-1].append(char)
            i += 1

        return ["".join(x) for x in output]

    def _tokenize_chinese_chars(self, text):
        """Adds whitespace around any CJK character."""
        output = []
        for char in text:
            cp = ord(char)
            if self._is_chinese_char(cp):
                output.append(" ")
                output.append(char)
                output.append(" ")
            else:
                output.append(char)
        return "".join(output)

    def _is_chinese_char(self, cp):
        """Checks whether CP is the codepoint of a CJK character."""
        # This defines a "chinese character" as anything in the CJK Unicode block:
        #   https://en.wikipedia.org/wiki/CJK_Unified_Ideographs_(Unicode_block)
        #
        # Note that the CJK Unicode block is NOT all Japanese and Korean characters,
        # despite its name. The modern Korean Hangul alphabet is a different block,
        # as is Japanese Hiragana and Katakana. Those alphabets are used to write
        # space-separated words, so they are not treated specially and handled
        # like the all of the other languages.
        if (
            (cp >= 0x4E00 and cp <= 0x9FFF)
            or (cp >= 0x3400 and cp <= 0x4DBF)
            or (cp >= 0x20000 and cp <= 0x2A6DF)  #
            or (cp >= 0x2A700 and cp <= 0x2B73F)  #
            or (cp >= 0x2B740 and cp <= 0x2B81F)  #
            or (cp >= 0x2B820 and cp <= 0x2CEAF)  #
            or (cp >= 0xF900 and cp <= 0xFAFF)
            or (cp >= 0x2F800 and cp <= 0x2FA1F)  #
        ):  #
            return True

        return False

    def _clean_text(self, text):
        """Performs invalid character removal and whitespace cleanup on text."""
        output = []
        for char in text:
            cp = ord(char)
            if cp == 0 or cp == 0xFFFD or _is_control(char):
                continue
            if _is_whitespace(char):
                output.append(" ")
            else:
                output.append(char)
        return "".join(output)


class WordpieceTokenizer(object):
    """Runs WordPiece tokenization."""

    def __init__(self, vocab, unk_token, max_input_chars_per_word=100):
        self.vocab = vocab
        self.unk_token = unk_token
        self.max_input_chars_per_word = max_input_chars_per_word

    def tokenize(self, text):
        """Tokenizes a piece of text into its word pieces.

        This uses a greedy longest-match-first algorithm to perform tokenization
        using the given vocabulary.

        For example:
          input = "unaffable"
          output = ["un", "##aff", "##able"]

        Args:
          text: A single token or whitespace separated tokens. This should have
            already been passed through `BasicTokenizer`.

        Returns:
          A list of wordpiece tokens.
        """

        output_tokens = []
        for token in whitespace_tokenize(text):
            chars = list(token)
            if len(chars) > self.max_input_chars_per_word:
                output_tokens.append(self.unk_token)
                continue

            is_bad = False
            start = 0
            sub_tokens = []
            while start < len(chars):
                end = len(chars)
                cur_substr = None
                while start < end:
                    substr = "".join(chars[start:end])
                    if start > 0:
                        substr = "##" + substr
                    if substr in self.vocab:
                        cur_substr = substr
                        break
                    end -= 1
                if cur_substr is None:
                    is_bad = True
                    break
                sub_tokens.append(cur_substr)
                start = end

            if is_bad:
                output_tokens.append(self.unk_token)
            else:
                output_tokens.extend(sub_tokens)
        return output_tokens


def _is_whitespace(char):
    """Checks whether `chars` is a whitespace character."""
    # \t, \n, and \r are technically contorl characters but we treat them
    # as whitespace since they are generally considered as such.
    if char == " " or char == "\t" or char == "\n" or char == "\r":
        return True
    cat = unicodedata.category(char)
    if cat == "Zs":
        return True
    return False


def _is_control(char):
    """Checks whether `chars` is a control character."""
    # These are technically control characters but we count them as whitespace
    # characters.
    if char == "\t" or char == "\n" or char == "\r":
        return False
    cat = unicodedata.category(char)
    if cat.startswith("C"):
        return True
    return False


def _is_punctuation(char):
    """Checks whether `chars` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if (cp >= 33 and cp <= 47) or (cp >= 58 and cp <= 64) or (cp >= 91 and cp <= 96) or (cp >= 123 and cp <= 126):
        return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
        return True
    return False



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="transformers_keras",
    version="0.1.2",
    description="Transformer-based models implemented in tensorflow 2.x(Keras)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/luozhouyang/transformers-keras",
    author="ZhouYang Luo",
    author_email="zhouyang.luo@gmail.com",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
        "jieba",
        "matplotlib",
    ],
    extras_require={
        "tf": ["tensorflow>=2.0.1"],
        "tf-gpu": ["tensorflow-gpu>=2.0.1"],
    },
    license="Apache Software License",
    classifiers=(
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    )
)



from .bert_tokenizer import BertAbstractTokenizer, BertVocabBasedTokenizer, BertDefaultTokenizer
from .transformer_tokenizer import TransformerAbstractTokenizer
from .transformer_tokenizer import TransformerVocabBasedTokenizer
from .transformer_tokenizer import TransformerDefaultTokenizer



import tensorflow as tf

from .abstract_dataset_builder import AbstractDatasetBuilder


class BertTFRecordDatasetBuilder(AbstractDatasetBuilder):

    def __init__(self, max_sequence_length=512, max_predictions_per_seq=20, record_option=None, **kwargs):
        super().__init__(**kwargs)
        self.record_option = record_option
        self.max_sequence_length = max_sequence_length
        self.max_predictions_per_seq = max_predictions_per_seq

    def build_train_dataset(self, train_record_files, **kwargs):
        dataset = tf.data.TFRecordDataset(train_record_files, compression_type=self.record_option)
        dataset = dataset.repeat(self.train_repeat_count)
        dataset = dataset.shuffle(
            buffer_size=self.train_shuffle_buffer_size,
            seed=self.train_shuffle_seed,
            reshuffle_each_iteration=self.train_reshuffle_each_iteration)
        dataset = dataset.map(self._parse_example_fn, num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.batch(
            self.train_batch_size, drop_remainder=self.train_drop_remainder
        ).prefetch(self.prefetch_size)
        dataset = dataset.map(lambda x, y, z, p, l: ((x, y, z), (p, l)))
        return dataset

    def build_valid_dataset(self, valid_record_files, **kwargs):
        if valid_record_files is None:
            return None
        dataset = tf.data.TFRecordDataset(valid_record_files, compression_type=self.record_option)
        dataset = dataset.repeat(self.valid_repeat_count)
        dataset = dataset.shuffle(
            buffer_size=self.valid_shuffle_buffer_size,
            seed=self.valid_shuffle_seed,
            reshuffle_each_iteration=self.valid_reshuffle_each_iteration
        )
        dataset = dataset.map(self._parse_example_fn, num_parallel_calls=self.num_parallel_calls)
        dataset = dataset.batch(
            self.valid_batch_size, drop_remainder=self.valid_drop_remainder
        ).prefetch(self.prefetch_size)
        dataset = dataset.map(lambda x, y, z, p, l: ((x, y, z), (p, l)))
        return dataset

    def build_predict_dataset(self, predict_record_files, **kwargs):
        dataset = tf.data.TFRecordDataset(predict_record_files, compression_type=self.record_option)
        dataset = dataset.repeat(self.predict_repeat_count)
        dataset = dataset.map(self._parse_example_fn)
        dataset = dataset.batch(
            self.predict_batch_size, drop_remainder=self.predict_drop_remainder
        ).prefetch(self.prefetch_size)
        dataset = dataset.map(lambda x, y, z, p, l: ((x, y, z), None))
        return dataset

    def _parse_example_fn(self, record):
        """Parse tfrecord to example. Change this parse process according to your record files format."""
        MAX_SEQ_LEN = self.max_sequence_length
        MAX_PREDICTIONS_PER_SEQ = self.max_predictions_per_seq
        name_to_features = {
            'original_ids': tf.io.FixedLenFeature([MAX_SEQ_LEN], tf.int64),
            'input_ids': tf.io.FixedLenFeature([MAX_SEQ_LEN], tf.int64),
            'input_mask': tf.io.FixedLenFeature([MAX_SEQ_LEN], tf.int64),
            'segment_ids': tf.io.FixedLenFeature([MAX_SEQ_LEN], tf.int64),
            'masked_lm_positions': tf.io.FixedLenFeature([MAX_PREDICTIONS_PER_SEQ], tf.int64),
            'masked_lm_ids': tf.io.FixedLenFeature([MAX_PREDICTIONS_PER_SEQ], tf.int64),
            'masked_lm_weights': tf.io.FixedLenFeature([MAX_PREDICTIONS_PER_SEQ], tf.float32),
            'next_sentence_labels': tf.io.FixedLenFeature([1], tf.int64),
        }

        example = tf.io.parse_single_example(record, name_to_features)
        # tf.Example only supports tf.int64, but the TPU only supports tf.int32.
        # So cast all int64 to int32.
        for name in list(example.keys()):
            t = example[name]
            if t.dtype == tf.int64:
                t = tf.cast(t, tf.int32)
            example[name] = t

        # extract needed values from example to matching network's inputs and outputs
        # features = {
        #     'input_ids': example.get('input_ids', None),
        #     'input_mask': example.get('input_mask', None),
        #     'segment_ids': example.get('segment_ids', None),
        # }
        # labels = {
        #     'predictions': example['original_ids'],
        #     'relations': tf.one_hot(example['next_sentence_labels'], 2)
        # }
        input_ids = example['input_ids']
        input_mask = example['input_mask']
        segment_ids = example['segment_ids']
        predictions = example['original_ids']
        relations = example['next_sentence_labels']
        return (input_ids, input_mask, segment_ids, predictions, relations)
